//
//  Painter.java
//  design by numbers 01
//
//  Created by Harry Vermeulen on 4/21/07.
//  Copyright 2007 __MyCompanyName__. All rights reserved.
//

import com.lowagie.text.*;
import com.lowagie.text.pdf.*;
import java.io.FileOutputStream;
import java.io.IOException;

import com.lowagie.text.Document;
import com.lowagie.text.Font;
import com.lowagie.text.Paragraph;
import com.lowagie.text.pdf.BaseFont;
import com.lowagie.text.pdf.PdfWriter;
import com.lowagie.text.*;



public class Painter {
	//Declarations
	int marge = 30;
	Document document = new Document(PageSize.A0, marge, marge, marge, marge);
	PdfWriter writer;
	PdfContentByte cb;
	PdfContentByte cbh;
	Rectangle r;
	String fileName = "untitled.pdf";
	NumberGenerator n = new NumberGenerator();
	Image eye;
	BaseFont JapaneseBaseFont;
	Font FontJapanese;
	HiraganaLib h = new HiraganaLib();
	int neckCurve = 200;

	//Constructor
	public Painter(String fileName) {
		try {
			writer = PdfWriter.getInstance(document, new FileOutputStream(fileName));
			writer.setStrictImageSequence(true);
			document.open();
			System.out.println("Document opened.");
			cb = writer.getDirectContentUnder();
			cbh = writer.getDirectContent();
			JapaneseBaseFont = BaseFont.createFont("SimSun.ttf", BaseFont.IDENTITY_H, BaseFont.NOT_EMBEDDED);
		}
		catch(DocumentException de) {
			System.err.println(de.getMessage());
		}
		catch(IOException ioe) {
			System.err.println(ioe.getMessage());
		}
		FontJapanese = new Font(JapaneseBaseFont, 36);
	}

	void drawLine(Point start, Point end, float opacity, int width, CMYKColor color) {
		//link contentbyte to writer
		PdfContentByte c = new PdfContentByte(writer);
		
		//line settings, opacity, width, color
		PdfGState g = new PdfGState();
		g.setStrokeOpacity(opacity / 100);
		c.setGState(g);
		c.setLineWidth(width);
		c.setCMYKColorStroke((int)(color.getCyan() * 255), 
							 (int)(color.getMagenta() * 255), 
							 (int)(color.getYellow() * 255), 
							 (int)(color.getBlack() * 255));
		
		//defining the line
		c.moveTo(start.x(), start.y());
		c.lineTo(end.x(), end.y());
		
		//drawing the line
		c.closePath();
		c.stroke();
		cb.add(c);
	}
	
	void drawBezier(Point start, Point end, Point bezier1, Point bezier2, float opacity, int width, CMYKColor color) { 
		//link contentbyte to writer
		PdfContentByte c = new PdfContentByte(writer);
		
		//line settings, opacity, width, color
		PdfGState g = new PdfGState();
		g.setStrokeOpacity(opacity / 100);
		c.setGState(g);
		c.setLineWidth(width);
		c.setCMYKColorStroke((int)(color.getCyan() * 255), 
							 (int)(color.getMagenta() * 255), 
							 (int)(color.getYellow() * 255), 
							 (int)(color.getBlack() * 255));

		c.moveTo(start.x(), start.y());
		c.curveTo(bezier1.x(), bezier1.y(), bezier2.x(), bezier2.y(),end.x(), end.y());
	
		c.stroke();
		cb.add(c);
	}
	
	void drawEye(Point position, int size, float eyeSize, CMYKColor colorOrgans, CMYKColor colorSpine, CMYKColor colorEyeBrow) {		
		PdfContentByte c = new PdfContentByte(writer);
		
		int a = (int) position.x();
		int b = (int) position.y();
		float ex = n.erc(a, b, size).x();
		float ey = n.erc(a, b, size).y();
		
		try {
			eye = Image.getInstance("eye.png");
		}
		catch(IOException ioe) {
            System.err.println(ioe.getMessage());
        }
		catch(BadElementException exe) {
            System.err.println(exe.getMessage());
        }

		PdfGState g = new PdfGState();
		g.setStrokeOpacity(0.1f);
		c.setGState(g);
		c.setCMYKColorStroke((int)(colorOrgans.getCyan() * 255), 
							 (int)(colorOrgans.getMagenta() * 255), 
							 (int)(colorOrgans.getYellow() * 255), 
							 (int)(colorOrgans.getBlack() * 255));
		
		for (int i = 0; i < (eyeSize / 5); i++) {
			c.setLineWidth((float) (Math.random() * 3));
			Point from = new Point((float) (a + (Math.random() - 0.5) * eyeSize),(float) (b + (Math.random() - 0.5) * eyeSize));
			Point to = new Point((float) (ex + (Math.random() - 0.5) * eyeSize),(float) (ey + (Math.random() - 0.5) * eyeSize));
			Point bezier1 = new Point((float) (a + (Math.random() - 0.5) * eyeSize + (Math.random() - 0.5) * neckCurve),
									  (float) (b + (Math.random() - 0.5) * eyeSize + (Math.random() - 0.5) * neckCurve));
			Point bezier2 = new Point((float) (a + (Math.random() - 0.5) * eyeSize + (Math.random() - 0.5) * neckCurve),
									  (float) (b + (Math.random() - 0.5) * eyeSize + (Math.random() - 0.5) * neckCurve));

			c.moveTo(from.x(), from.y());
			c.curveTo(bezier1.x(), bezier1.y(), bezier2.x(), bezier2.y(), to.x() + (eyeSize / 2), to.y() + (eyeSize / 2));
			c.stroke();
		}
		cb.add(c);

		g.setStrokeOpacity(0.1f);
		c.setGState(g);
		c.setCMYKColorStroke((int)(colorSpine.getCyan() * 255), 
							 (int)(colorSpine.getMagenta() * 255), 
							 (int)(colorSpine.getYellow() * 255), 
							 (int)(colorSpine.getBlack() * 255));
		c.moveTo(a, b);
		c.lineTo(ex + (eyeSize / 2), ey + (eyeSize / 2));
		
		eye.setAbsolutePosition(ex, ey);
		eye.scalePercent(eyeSize);
		
		try {
			document.add(eye);
		}
		catch(DocumentException de) {
			System.err.println(de.getMessage());
		}
		
		c.stroke();
		cb.add(c);
	}
	
	void drawRandomName(Point position, int size, int organs) {
		String nameHiragana = "";
		String nameRoman = "";
		for (int i = 0; i < (organs / 5) + 1; i++) {
			int x = (int) (Math.random() * h.hiragana.length);
			nameHiragana = nameHiragana + h.hiragana[x];
			nameRoman = nameRoman + h.roman[x];
		}
		
		cb.setRGBColorFillF(0f, 0f, 0f);
		cb.beginText();
		cb.setFontAndSize(JapaneseBaseFont, 18);
		cb.setTextMatrix(position.x() - size, position.y() - (size * 2));
		cb.showText(nameHiragana);
		cb.endText();
		
		cb.beginText();
		cb.setFontAndSize(JapaneseBaseFont, 18);
		cb.setTextMatrix(position.x() - size, position.y() - ((size * 2) + 24));
		cb.showText(nameRoman);
		cb.endText();
	}
	
	void rectangle(float x, float y, float x2, float y2) {	
		PdfContentByte c = new PdfContentByte(writer);
		PdfGState g = new PdfGState();
		g.setStrokeOpacity(1f);
		c.setGState(g);
		c.setRGBColorFillF(.96f, .96f, .96f);
		c.rectangle(x, y, x2, y2);
		c.fill();
		cb.add(c);
	}
	
	float getPageWidth() {
		r = document.getPageSize();
		return r.width();
	}
	
	float getPageHeight() {
		r = document.getPageSize();
		return r.height();
	}
	
	int getMarge() {
		return marge;
	}
	
	void closeDocument() {
		document.close();
		System.out.println("Document closed.");
	}
}
